==> thirdparty-service-biz-0.0.9.3-20171201.122103-1-sources.jar: cn/com/duiba/notifycenter/service/NotifyHttpClientPool.java <==
package cn.com.duiba.notifycenter.service;

import cn.com.duiba.service.HttpClientFactory;
import cn.com.duiba.service.SlowRequestHandler;
import cn.com.duiba.service.ThreadPoolService;
import cn.com.duiba.tool.UrlUtils;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.concurrent.FutureCallback;
import org.apache.http.impl.nio.client.CloseableHttpAsyncClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 通知服务HTTP连接池
 */
@Service
public class NotifyHttpClientPool implements InitializingBean, DisposableBean {

	private static final Logger log = LoggerFactory.getLogger(NotifyHttpClientPool.class);

	private CloseableHttpAsyncClient httpClient;

	private Map<Long, AtomicInteger> runningStat = new ConcurrentHashMap<>();

	@Autowired
	private ThreadPoolService threadPoolService;
	@Autowired
	private SlowRequestHandler slowRequestHandler;

	/**
	 * 执行HTTP异步请求
	 *
	 * @param appId
	 * @param request
	 * @param callback
	 */
	public void execute(Long appId, HttpUriRequest request, FutureCallback<HttpResponse> callback) {
		try {
			long s = System.currentTimeMillis();
			AtomicInteger count = runningStat.get(appId);
			if (count == null) {
				count = new AtomicInteger(1);
				runningStat.put(appId, count);
			} else {
				count.incrementAndGet();
			}
            request.setHeader("User-Agent","Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1");
            CallbackProcesser process = new CallbackProcesser(appId, request, callback);

			String reqUrl = UrlUtils.buildUrl(request.getURI());
			if (slowRequestHandler.isSlow(reqUrl)) {
				log.info("url [{}], add to queue, appId={}", reqUrl, appId);
				slowRequestHandler.addQueue(process);
				return;
			}

			if (!httpClient.isRunning()) {
				httpClient.close();
				start();
			}
            httpClient.execute(process.getRequest(), process);
			long e = System.currentTimeMillis();
			long t = e - s;
			if (t > 1000) {
				log.warn("notify httpClent.execute > {}ms, url={}, appId={}", t, reqUrl, appId);
				if (t>2000) {
					slowRequestHandler.addSlow(reqUrl, (int)(t/1000L));
				}
			}
		} catch (Exception e) {
			callback.failed(e);
			log.error("execute:", e);
		}
	}

	/**
	 * 提交HTTP异步请求
	 * 
	 * @param appId
	 * @param request
	 * @param callback
	 */
	public void submit(final Long appId, final HttpUriRequest request, final FutureCallback<HttpResponse> callback) {
		if (threadPoolService.canSubmit()) {
			threadPoolService.submit(new Runnable() {
				@Override
				public void run() {
					execute(appId, request, callback);
				}
			});
		} else {
			execute(appId, request, callback);
		}
	}

	/**
	 * 连接池详细信息
	 * 
	 * @return
	 */
	public JSONObject dumpDetail() {
		Map<Long, Object> map = new HashMap<>();
		for (Map.Entry<Long, AtomicInteger> entry : runningStat.entrySet()) {
			if (entry.getValue().intValue() > 0) {
				map.put(entry.getKey(), entry.getValue());
			}
		}
		JSONObject o = new JSONObject();
		o.put("runningStat", map);
		return o;
	}

	/**
	 * 连接池大小
	 * 
	 * @return
	 */
	public int dumpSize() {
		int total = 0;
		for (AtomicInteger a : runningStat.values()) {
			total += a.get();
		}
		return total;
	}

	/**
	 * 通知请求回调类
	 */
	public class CallbackProcesser implements FutureCallback<HttpResponse> {
		private HttpUriRequest request;
		private FutureCallback<HttpResponse> callback;
		private Long appId;

		/**
		 * CallbackProcesser
		 * 
		 * @param appId
		 * @param request
		 * @param callback
		 */
		public CallbackProcesser(Long appId, HttpUriRequest request, FutureCallback<HttpResponse> callback) {
			this.request = request;
			this.callback = callback;
			this.appId = appId;
		}

		@Override
		public void completed(HttpResponse result) {
			try {
				callback.completed(result);
			} finally {
				runningStat.get(appId).decrementAndGet();
			}
		}

		@Override
		public void failed(Exception ex) {
			try {
				callback.failed(ex);
			} finally {
				runningStat.get(appId).decrementAndGet();
			}
		}

		@Override
		public void cancelled() {
			try {
				callback.cancelled();
			} finally {
				runningStat.get(appId).decrementAndGet();
			}
		}

		/**
		 * getRequest
		 * 
		 * @return
		 */
		public HttpUriRequest getRequest() {
			return request;
		}

		public Long getAppId() {
			return appId;
		}
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		start();
	}

	@Override
	public void destroy() throws Exception {
		if (httpClient != null && httpClient.isRunning()) {
			httpClient.close();
			log.info("Notify httpClient closed");
		}
	}

	/**
	 * 启动HTTP服务
	 */
	private synchronized void start() {
		if (httpClient != null && httpClient.isRunning()) {
			return;
		}
		httpClient = HttpClientFactory.newCloseableHttpAsyncClient();
		log.info("Notify httpClient started");
	}

}

==> thirdparty-service-biz-0.0.9.10-20180319.064614-2-sources.jar: cn/com/duiba/notifycenter/service/NotifyHttpClientPool.java <==
package cn.com.duiba.notifycenter.service;

import cn.com.duiba.service.HttpClientFactory;
import cn.com.duiba.service.SlowRequestHandler;
import cn.com.duiba.service.ThreadPoolService;
import cn.com.duiba.tool.UrlUtils;
import com.alibaba.fastjson.JSONObject;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.concurrent.FutureCallback;
import org.apache.http.impl.nio.client.CloseableHttpAsyncClient;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.DisposableBean;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * 通知服务HTTP连接池
 */
@Service
public class NotifyHttpClientPool implements InitializingBean, DisposableBean {

	private static final Logger log = LoggerFactory.getLogger(NotifyHttpClientPool.class);

	private CloseableHttpAsyncClient httpClient;

	private Map<Long, AtomicInteger> runningStat = new ConcurrentHashMap<>();

	@Autowired
	private ThreadPoolService threadPoolService;
	@Autowired
	private SlowRequestHandler slowRequestHandler;

	private final BlockingQueue<CallbackProcesser> latterNotifyQueue = new LinkedBlockingQueue<>(50000);
	private final ExecutorService executorService = Executors.newSingleThreadExecutor(new ThreadFactory() {
		@Override
		public Thread newThread(Runnable r) {
			return new Thread(r, "LatterNotifyQueueProcess");
		}
	});
	private volatile boolean started = true;

	/**
	 * 执行HTTP异步请求
	 *
	 * @param appId
	 * @param request
	 * @param callback
	 */
	public void execute(Long appId, HttpUriRequest request, FutureCallback<HttpResponse> callback) {
		try {
			long s = System.currentTimeMillis();
			AtomicInteger count = runningStat.get(appId);
			if (count == null) {
				count = new AtomicInteger(1);
				runningStat.put(appId, count);
			} else {
				count.incrementAndGet();
			}
            request.setHeader("User-Agent","Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1");
            CallbackProcesser process = new CallbackProcesser(appId, request, callback);

			String reqUrl = UrlUtils.buildUrl(request.getURI());
			if(slowRequestHandler.isBlackHost(reqUrl)){
				log.info("{} is black host", reqUrl);
				return;
			}
			if (slowRequestHandler.isSlow(reqUrl)) {
				log.info("url [{}], add to queue, appId={}", reqUrl, appId);
				slowRequestHandler.addQueue(process);
				return;
			}

			if (!httpClient.isRunning()) {
				httpClient.close();
				start();
			}
            httpClient.execute(process.getRequest(), process);
			long e = System.currentTimeMillis();
			long t = e - s;
			if (t > 1000) {
				log.warn("notify httpClent.execute > {}ms, url={}, appId={}", t, reqUrl, appId);
				if (t>2000) {
					slowRequestHandler.addSlow(reqUrl, (int)(t/1000L));
				}
			}
		} catch (Exception e) {
			callback.failed(e);
			log.error("execute:", e);
		}
	}

	/**
	 * 提交HTTP异步请求
	 * 
	 * @param appId
	 * @param request
	 * @param callback
	 */
	public void submit(final Long appId, final HttpUriRequest request, final FutureCallback<HttpResponse> callback) {
		if (threadPoolService.canSubmit()) {
			threadPoolService.submit(new Runnable() {
				@Override
				public void run() {
					execute(appId, request, callback);
				}
			});
		} else {
			try {
				latterNotifyQueue.add(new CallbackProcesser(appId, request, callback));
			} catch (IllegalStateException e) {
				// 队列满了,任务取消
				log.warn("latterNotifyQueue is temporarily full",e);
				callback.cancelled();
			}
		}
	}

	/**
	 * 连接池详细信息
	 * 
	 * @return
	 */
	public JSONObject dumpDetail() {
		Map<Long, Object> map = new HashMap<>();
		for (Map.Entry<Long, AtomicInteger> entry : runningStat.entrySet()) {
			if (entry.getValue().intValue() > 0) {
				map.put(entry.getKey(), entry.getValue());
			}
		}
		JSONObject o = new JSONObject();
		o.put("runningStat", map);
		return o;
	}

	/**
	 * 连接池大小
	 * 
	 * @return
	 */
	public int dumpSize() {
		int total = 0;
		for (AtomicInteger a : runningStat.values()) {
			total += a.get();
		}
		return total;
	}

	/**
	 * 通知请求回调类
	 */
	public class CallbackProcesser implements FutureCallback<HttpResponse> {
		private HttpUriRequest request;
		private FutureCallback<HttpResponse> callback;
		private Long appId;

		/**
		 * CallbackProcesser
		 * 
		 * @param appId
		 * @param request
		 * @param callback
		 */
		public CallbackProcesser(Long appId, HttpUriRequest request, FutureCallback<HttpResponse> callback) {
			this.request = request;
			this.callback = callback;
			this.appId = appId;
		}

		@Override
		public void completed(HttpResponse result) {
			try {
				callback.completed(result);
			} finally {
				runningStat.get(appId).decrementAndGet();
			}
		}

		@Override
		public void failed(Exception ex) {
			try {
				callback.failed(ex);
			} finally {
				runningStat.get(appId).decrementAndGet();
			}
		}

		@Override
		public void cancelled() {
			try {
				callback.cancelled();
			} finally {
				runningStat.get(appId).decrementAndGet();
			}
		}

		/**
		 * getRequest
		 * 
		 * @return
		 */
		public HttpUriRequest getRequest() {
			return request;
		}

		public Long getAppId() {
			return appId;
		}
	}

	@Override
	public void afterPropertiesSet() throws Exception {
		start();
		executorService.execute(new Runnable() {
			@Override
			public void run() {
				while (started) {
					try{
						if (!threadPoolService.canSubmit()) {
							TimeUnit.MILLISECONDS.sleep(200);
							continue;
						}
						CallbackProcesser callbackProcesser =latterNotifyQueue.take();
						final Long appId = callbackProcesser.getAppId();
						final HttpUriRequest request = callbackProcesser.getRequest();
						final FutureCallback<HttpResponse> callback = callbackProcesser.callback;
						threadPoolService.submit(new Runnable() {
							@Override
							public void run() {
								execute(appId, request, callback);
							}
						});
					}catch (InterruptedException e) {
						log.warn("Interrupted",e);
					}catch (Exception e) {
						log.warn("",e);
					}
				}
			}
		});
	}

	@Override
	public void destroy() throws Exception {
		started = false;
		executorService.shutdown();
		if (httpClient != null && httpClient.isRunning()) {
			httpClient.close();
			log.info("Notify httpClient closed");
		}
	}

	/**
	 * 启动HTTP服务
	 */
	private synchronized void start() {
		if (httpClient != null && httpClient.isRunning()) {
			return;
		}
		httpClient = HttpClientFactory.newNotifyHttpAsyncClient();
		log.info("Notify httpClient started");
	}

}
